import xml.etree.ElementTree as ET
import ldap
import ldap.modlist as modlist
import json
import urllib
import urllib2
import urlparse

from mellon.models import UserSAMLIdentifier

base = "ou=People,dc=entrouvert,dc=lan"
scope = ldap.SCOPE_SUBTREE
pocform = 'traitement'

def ldap_init():
    # The server's hostname:
    server = "spare.entrouvert.lan"
    # Admin DN:
    who = "cn=admin,dc=entrouvert,dc=lan"
    # Credentials: XXX
    cred = "test"

    l = ldap.open(server)
    #l.protocol_version = ldap.VERSION3 # Unnecessary here
    if l.simple_bind(who, cred):
        return l
    else:
        return None

def ldap_terminate(l):
    l.unbind()

def ldap_is_in_directory(id):
    # The way we derive UIDs from the id dict
    #   is defined in ldap_craft_uid:
    filter = "uid="+ldap_craft_uid(id)

    l = ldap_init()

    # Look for a user possessing the same uid:
    res = l.search(base, scope, filter, None)
    rtype, rdata = l.result(res, 0)

    ldap_terminate(l)

    # Does this user already is in the LDAP directory ?
    return 1 if rdata else 0

def ldap_max_uidnumber():
    l = ldap_init()

    # We need to iterate the whole user list in
    #   the default base DN:
    res = l.search(base, scope, '(uid=*)', None)
    rtype, rdata = l.result(res, 0)

    # Let's assume UIDs are strictly positive numbers:
    uidNumber = 0

    # rdata is non-empty as long as there are still
    #   some entries to iterate over:
    while rdata :
        un = int(rdata[0][1]['uidNumber'][0])
        # Let's get the highest UID:
        if un > uidNumber:
            uidNumber = un
        rtype, rdata = l.result(res, 0)

    ldap_terminate(l)
    return uidNumber

def craft_uid(id):
    return (id['prenom'][0]+id['nom']).lower()

def ldap_craft_uid(id):
    return craft_uid(id)

def ldap_add_entry(id):
    # uid and uidNumber on-the-fly production:
    uidNumber = ldap_max_uidnumber()+1
    #uid = ldap_craft_uid(id)

    # The to-be-added entry DN.
    #   the "dn: " prefix musn't appear here:
    dn = 'uid='+id['nameid']+","+base

    # All the entry attributes can be defind in a dictionary as below:
    addmod = {}
    addmod['objectClass'] = ['inetOrgPerson', 'organizationalPerson', 'person', 'posixAccount', 'top']
    addmod['cn'] = id['prenom']+" "+id['nom']
    addmod['uid'] = id['nameid']
    addmod['uidNumber'] = str(uidNumber)
    addmod['gidNumber'] = '1000'
    addmod['sn'] = id['nom']
    addmod['homeDirectory'] = "/home/"+id['nameid']

    # From here we get an addModlist intelligible to python-ldap:
    ldif = modlist.addModlist(addmod)
    l = ldap_init()

    # Do not forget the entry DN:
    ret = l.add_s(dn, ldif)

    ldap_terminate(l)

    # Temporary debug output
    return str(dn)+"\n"+str(ldif)+"\n"+str(ret)
    #return str(addmod)

def saml_collect_data():
    form = {}

    # Build the SSO operation summary:
    form['prenom'] = str(UserSAMLIdentifier.objects.last().user.first_name)
    form['nom'] = str(UserSAMLIdentifier.objects.last().user.last_name)
    form['email'] = str(UserSAMLIdentifier.objects.last().user.email)
    #form['password'] = UserSAMLIdentifier.objects.last().user.password # dirty
    form['nameid'] = craft_uid(form) # XXX redundant information

    return form

def wcs_submit(id):
    # First version:
    #   implicit static schema definition
    #   TODO runtime REST schema query from wcs

    opener = urllib2.build_opener(urllib2.HTTPHandler)

    # Generate a JSON to bind against the wcs ReST API
    form = {}
    form["data"] = id # see wcs documentation

    # Add Passerelle validation URL:
    #form[''] = ...

    data = json.dumps(form)

    posturl = 'http://wcs.example.com/api/formdefs/'+pocform+'/submit'

    req = urllib2.Request(posturl, data)
    req.add_header("Content-Type", "application/json")
    req.add_header("Accept", "application/json")

    feedback = opener.open(req)

    return 0
