from django.shortcuts import render, redirect
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import FormView
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import logout as auth_logout
from django.contrib.auth import views as auth_views
from django.core.exceptions import ObjectDoesNotExist, PermissionDenied
from django.core.urlresolvers import reverse
from django.db import transaction
from django.http import (Http404, HttpResponse, HttpResponseRedirect,
        HttpResponsePermanentRedirect)
from django.shortcuts import render, resolve_url
from django.template import RequestContext, loader
from django.utils.translation import ugettext as _
from django.forms.widgets import Media
from mellon.utils import get_idps
from inspect import getmembers

from .forms import UnitAssignmentForm
from .models import Unit
from .decorators import user_not_in_ldap
from .utils import ldap_is_in_directory, ldap_add_entry, saml_collect_data, wcs_submit

import django
import pdb

# TODOs
#   export global config variables in settings.py

# Create your views here.
wcs_fields = ['prenom', 'nom', 'email', 'nameid', 'unit']


# DEPRECATED
@csrf_exempt
def index(request):

    # SAML assertions collected by django-mellon:
    data = saml_collect_data()

    # Do no add an already-existing user
    if ldap_is_in_directory(data):
        return HttpResponse("Utilisateur deja present")
    else:
        # We need to format the entry before adding it
        #   in the LDAP directory
        ldap_add_entry(data)

    # Submit user info to the wcs REST API:
    return wcs_submit(data)

# DEPRECATED
@csrf_exempt
def oldunit(request, data = []):
    form = UnitAssignmentForm(request.POST or None, initial=data) #TODO LDAPGetUnitList
    flag = 0
    if form.is_valid():
        data['unit'] = form.cleaned_data['unit']
        flag = 1
    return render(request, 'unit_form.html', locals())

class UnitFormView(FormView):
    form_class = UnitAssignmentForm
    template_name = 'unit_form.html'
    success_url = '/unit' # Mandatory ?

    @user_not_in_ldap
    def get(self, request, *args, **kwargs):
        # SAML assertions collected by django-mellon:
        data = saml_collect_data() # Reentrancy issue

        # Do no add an already-existing user
        if ldap_is_in_directory(data):
            return HttpResponse("Utilisateur deja present")
        else:
            prenom = data['prenom']
            nom = data['nom']
            form = self.form_class(initial=data)
            return render(request, 'unit_form.html', locals())

    def post(self, request, *args, **kwargs):
        data = {}
        for field in wcs_fields:
            data[field] = str(request.POST[field])
        #return HttpResponse(str(data))
        #return HttpResponse(str(request.POST.items()))
        wcs_submit(data)
        return render(request, 'unit_sent.html')
        #return render(request, 'unit_sent.html', locals())

## DEPRECATED
#def saml_parse(saml_assert):
#    # XML namespaces defined in the sample SAML assertion:
#    ns = {'saml' : 'urn:oasis:names:tc:SAML:2.0:assertion',
#          'xs' :  'http://www.w3.org/2001/XMLSchema',
#          'xsi' : 'http://www.w3.org/2001/XMLSchema-instance'}
#
#    # Identity dictionary
#    id = {'first_name' : '',
#          'last_name' : '',
#          'urn:oid:1.3.6.1.4.1.5923.1.1.1.6' : ''} #EPPN
#
#    tree = ET.fromstring(saml_assert)
#
#    # Get all saml:Attribute nodes
#    #   i.e. nodes under saml:AttributeStatement
#    for node in tree.find('saml:AttributeStatement', ns):
#        # We need to fill the dictionary
#        if id.has_key(node.get('Name')):
#            id[node.get('Name')] = node.find('saml:AttributeValue',
#            ns).text.strip()
#
#    # The saml:NameID node is located under saml:Subject:
#    subject = tree.find('saml:Subject', ns)
#    id['NameID'] = subject.find('saml:NameID', ns).text.strip()
#
#    return id

@csrf_exempt
def login(request, *args, **kwargs):
    if any(get_idps()):
        if not 'next' in request.GET:
            return HttpResponseRedirect(resolve_url('mellon_login'))
        return HttpResponseRedirect(resolve_url('mellon_login') + '?next='
                                    + urllib.quote(request.GET.get('next')))
    return auth_views.login(request, *args, **kwargs)

@csrf_exempt
def logout(request, next_page=None):
    if any(get_idps()):
        return HttpResponseRedirect(resolve_url('mellon_logout'))
    auth_logout(request)
    if next_page is not None:
        next_page = resolve_url(next_page)
    else:
        #next_page = '/'
        next_page = '/foo'
    return HttpResponseRedirect(next_page)
